//
//  TangRtcChannel.h
//  TangRtcChannel
//
//  Copyright (c) 2019 Tang. All rights reserved.
//

#import "TangRtcEngineKit.h"

@class TangRtcChannelMediaOptions;

@protocol TangRtcChannelDelegate;

@interface TangRtcEngineKit ()

- (TangRtcChannel* _Nullable)createRtcChannel:(NSString* _Nonnull)channelId;

@end

__attribute__((visibility("default"))) @interface TangRtcChannel : NSObject

- (int)destroy;

- (void)setRtcChannelDelegate:(id<TangRtcChannelDelegate> _Nullable)channelDelegate;

- (NSString* _Nullable)getChannelId;

- (NSString* _Nullable)getCallId;

- (TangConnectionStateType)getConnectionState;

- (int)joinChannelByToken:(NSString* _Nullable)token info:(NSString* _Nullable)info uid:(NSUInteger)uid options:(TangRtcChannelMediaOptions* _Nonnull)options;

- (int)joinChannelByUserAccount:(NSString* _Nonnull)userAccount token:(NSString* _Nullable)token options:(TangRtcChannelMediaOptions* _Nonnull)options;

- (int)leaveChannel;

- (int)setAVSyncSource:(NSString* _Nullable)channelId uid:(NSUInteger)uid;

- (int)muteLocalAudioStream:(BOOL)mute;

- (int)muteLocalVideoStream:(BOOL)mute;

- (int)publish;

- (int)unpublish;

- (int)setClientRole:(TangClientRole)role;

- (int)renewToken:(NSString* _Nonnull)token;

- (int)setEncryptionSecret:(NSString* _Nullable)secret;

- (int)setEncryptionMode:(NSString* _Nullable)encryptionMode;

- (int)setRemoteUserPriority:(NSUInteger)uid type:(TangUserPriority)userPriority;

- (int)setRemoteVoicePosition:(NSUInteger)uid pan:(double)pan gain:(double)gain;

- (int)setRemoteRenderMode:(NSUInteger)uid renderMode:(TangVideoRenderMode)renderMode mirrorMode:(TangVideoMirrorMode)mirrorMode;

- (int)setDefaultMuteAllRemoteAudioStreams:(BOOL)mute;

- (int)setDefaultMuteAllRemoteVideoStreams:(BOOL)mute;

- (int)muteRemoteAudioStream:(NSUInteger)uid mute:(BOOL)mute;

- (int)muteAllRemoteAudioStreams:(BOOL)mute;

- (int)muteRemoteVideoStream:(NSUInteger)uid mute:(BOOL)mute;

- (int)muteAllRemoteVideoStreams:(BOOL)mute;

- (int)setRemoteVideoStream:(NSUInteger)uid type:(TangVideoStreamType)streamType;

- (int)setRemoteDefaultVideoStreamType:(TangVideoStreamType)streamType;

- (int)removeInjectStreamUrl:(NSString* _Nonnull)url;

- (int)addPublishStreamUrl:(NSString* _Nonnull)url transcodingEnabled:(BOOL)transcodingEnabled;

- (int)removePublishStreamUrl:(NSString* _Nonnull)url;

//- (int)setLiveTranscoding:(LiveTranscoding* _Nullable)transcoding;

- (int)startRtmpStreamWithoutTranscoding:(NSString* _Nonnull)url;

- (int)createDataStream:(NSInteger* _Nonnull)streamId reliable:(BOOL)reliable ordered:(BOOL)ordered;

- (int)sendStreamMessage:(NSInteger)streamId data:(NSData* _Nonnull)data;

- (int)stopChannelMediaRelay;

#if defined(TARGET_OS_IPHONE) && TARGET_OS_IPHONE
- (int)enableRemoteSuperResolution:(NSUInteger)uid enabled:(BOOL)enabled;
- (BOOL)startSpeakerphone;
#endif

//- (void)setRemoteVideoRenderer:(id<TangVideoSinkProtocol> _Nullable)videoRenderer forUserId:(NSUInteger)userId;

//- (id<TangVideoSinkProtocol> _Nullable)remoteVideoRendererOfUserId:(NSUInteger)userId;

- (BOOL)setMediaMetadataDataSource:(id<TangMediaMetadataDataSource> _Nullable)metadataDataSource withType:(TangMetadataType)type;

- (BOOL)setMediaMetadataDelegate:(id<TangMediaMetadataDelegate> _Nullable)metadataDelegate withType:(TangMetadataType)type;



@end

@protocol TangRtcChannelDelegate <NSObject>
@optional

- (void)rtcChannel:(TangRtcChannel* _Nonnull)rtcChannel didOccurWarning:(TangWarningCode)warningCode;

- (void)rtcChannel:(TangRtcChannel* _Nonnull)rtcChannel didOccurError:(TangErrorCode)errorCode;

- (void)rtcChannelDidJoinChannel:(TangRtcChannel* _Nonnull)rtcChannel withUid:(NSUInteger)uid elapsed:(NSInteger)elapsed;

- (void)rtcChannelDidRejoinChannel:(TangRtcChannel* _Nonnull)rtcChannel withUid:(NSUInteger)uid elapsed:(NSInteger)elapsed;

- (void)rtcChannelDidLeaveChannel:(TangRtcChannel* _Nonnull)rtcChannel withStats:(TangChannelStats* _Nonnull)stats;

//- (void)rtcChannel:(TangRtcChannel* _Nonnull)rtcChannel didClientRoleChangeFailed:(TangClientRoleChangeFailedReason)reason currentRole:(TangClientRole)currentRole;

- (void)rtcChannel:(TangRtcChannel* _Nonnull)rtcChannel didClientRoleChanged:(TangClientRole)oldRole newRole:(TangClientRole)newRole;

- (void)rtcChannel:(TangRtcChannel* _Nonnull)rtcChannel didJoinedOfUid:(NSUInteger)uid elapsed:(NSInteger)elapsed;

- (void)rtcChannel:(TangRtcChannel* _Nonnull)rtcChannel didOfflineOfUid:(NSUInteger)uid reason:(TangUserOfflineReason)reason;

- (void)rtcChannel:(TangRtcChannel* _Nonnull)rtcChannel connectionChangedToState:(TangConnectionStateType)state reason:(TangConnectionChangedReason)reason;

//- (void)rtcChannel:(TangRtcChannel* _Nonnull)rtcChannel didProxyConnected:(NSUInteger)uid proxyType:(TangProxyType)proxyType localProxyIp:(NSString* _Nonnull)localProxyIp elapsed:(NSInteger)elapsed;

- (void)rtcChannelDidLost:(TangRtcChannel* _Nonnull)rtcChannel;

- (void)rtcChannel:(TangRtcChannel* _Nonnull)rtcChannel tokenPrivilegeWillExpire:(NSString* _Nonnull)token;

- (void)rtcChannelRequestToken:(TangRtcChannel* _Nonnull)rtcChannel;

- (void)rtcChannel:(TangRtcChannel* _Nonnull)rtcChannel activeSpeaker:(NSUInteger)speakerUid;

- (void)rtcChannel:(TangRtcChannel* _Nonnull)rtcChannel videoSizeChangedOfUid:(NSUInteger)uid size:(CGSize)size rotation:(NSInteger)rotation;

- (void)rtcChannel:(TangRtcChannel* _Nonnull)rtcChannel firstRemoteVideoFrameOfUid:(NSUInteger)uid size:(CGSize)size elapsed:(NSInteger)elapsed;

- (void)rtcChannel:(TangRtcChannel* _Nonnull)rtcChannel remoteVideoStateChangedOfUid:(NSUInteger)uid state:(TangVideoRemoteState)state reason:(TangVideoRemoteStateReason)reason elapsed:(NSInteger)elapsed;
;

//- (void)rtcChannel:(TangRtcChannel* _Nonnull)rtcChannel superResolutionEnabledOfUid:(NSUInteger)uid enabled:(BOOL)enabled reason:(TangSuperResolutionStateReason)reason;


- (void)rtcChannel:(TangRtcChannel* _Nonnull)rtcChannel remoteAudioStateChangedOfUid:(NSUInteger)uid state:(TangAudioRemoteState)state reason:(TangAudioRemoteStateReason)reason elapsed:(NSInteger)elapsed;

- (void)rtcChannel:(TangRtcChannel* _Nonnull)rtcChannel didLocalPublishFallbackToAudioOnly:(BOOL)isFallbackOrRecover;

- (void)rtcChannel:(TangRtcChannel* _Nonnull)rtcChannel didRemoteSubscribeFallbackToAudioOnly:(BOOL)isFallbackOrRecover byUid:(NSUInteger)uid;

- (void)rtcChannel:(TangRtcChannel* _Nonnull)rtcChannel reportRtcStats:(TangChannelStats* _Nonnull)stats;

- (void)rtcChannel:(TangRtcChannel* _Nonnull)rtcChannel networkQuality:(NSUInteger)uid txQuality:(TangNetworkQuality)txQuality rxQuality:(TangNetworkQuality)rxQuality;

- (void)rtcChannel:(TangRtcChannel* _Nonnull)rtcChannel remoteVideoStats:(TangRtcRemoteVideoStats* _Nonnull)stats;

- (void)rtcChannel:(TangRtcChannel* _Nonnull)rtcChannel remoteAudioStats:(TangRtcRemoteAudioStats* _Nonnull)stats;

- (void)rtcChannel:(TangRtcChannel* _Nonnull)rtcChannel rtmpStreamingChangedToState:(NSString* _Nonnull)url state:(TangRtmpStreamingState)state errorCode:(TangRtmpStreamingErrorCode)errorCode;


//- (void)rtcChannel:(TangRtcChannel* _Nonnull)rtcChannel rtmpStreamingEventWithUrl:(NSString* _Nonnull)url eventCode:(TangRtmpStreamingEvent)eventCode;


- (void)rtcChannelTranscodingUpdated:(TangRtcChannel* _Nonnull)rtcChannel;

#pragma mark Inject Stream URL Delegate Methods

- (void)rtcChannel:(TangRtcChannel* _Nonnull)rtcChannel streamInjectedStatusOfUrl:(NSString* _Nonnull)url uid:(NSUInteger)uid status:(TangInjectStreamStatus)status;

#pragma mark Stream Message Delegate Methods


- (void)rtcChannel:(TangRtcChannel* _Nonnull)rtcChannel receiveStreamMessageFromUid:(NSUInteger)uid streamId:(NSInteger)streamId data:(NSData* _Nonnull)data;

- (void)rtcChannel:(TangRtcChannel* _Nonnull)rtcChannel didOccurStreamMessageErrorFromUid:(NSUInteger)uid streamId:(NSInteger)streamId error:(NSInteger)error missed:(NSInteger)missed cached:(NSInteger)cached;

- (void)rtcChannel:(TangRtcChannel* _Nonnull)rtcChannel channelMediaRelayStateDidChange:(TangChannelMediaRelayState)state error:(TangChannelMediaRelayError)error;

- (void)rtcChannel:(TangRtcChannel* _Nonnull)rtcChannel didReceiveChannelMediaRelayEvent:(TangChannelMediaRelayEvent)event;

- (void)rtcChannel:(TangRtcChannel* _Nonnull)rtcChannel didAudioPublishStateChange:(TangStreamPublishState)oldState newState:(TangStreamPublishState)newState elapseSinceLastState:(NSInteger)elapseSinceLastState;


- (void)rtcChannel:(TangRtcChannel* _Nonnull)rtcChannel didVideoPublishStateChange:(TangStreamPublishState)oldState newState:(TangStreamPublishState)newState elapseSinceLastState:(NSInteger)elapseSinceLastState;

- (void)rtcChannel:(TangRtcChannel* _Nonnull)rtcChannel didAudioSubscribeStateChange:(NSUInteger)uid oldState:(TangStreamSubscribeState)oldState newState:(TangStreamSubscribeState)newState elapseSinceLastState:(NSInteger)elapseSinceLastState;

- (void)rtcChannel:(TangRtcChannel* _Nonnull)rtcChannel didVideoSubscribeStateChange:(NSUInteger)uid oldState:(TangStreamSubscribeState)oldState newState:(TangStreamSubscribeState)newState elapseSinceLastState:(NSInteger)elapseSinceLastState;

@end
